#!/usr/bin/python3
#  coding: utf-8
#
#       createVtkSourceFiles.py
#
#       meshのelement、surface、node group別にvtk形式で
#       fileを出力する。
#       出力先は、「vtkMeshData」folder内に出力。
#
#   20/06/15    新規作成
#      06/26    !EGROUPのvtkを追加
#      06/27    getVtkNodeData:offsetの値をi→i+1に修正
#               nodeGroupの最後のnodeが表示されない。
#               beamのnodeが表示されない。
#      06/30    path名表記の"/"をos.sepに修正
#

import os
import sys
import shutil

import pyFistr
import convertMesh as cm

currDir = ""
SaveFolder = "vtkMeshData"          #vtkFileを保存するfolder名

#
#  getMeshHeaderData
#---------------------
def getMeshHeaderData():
    """ meshDataを取得"""
    lines = pyFistr.readFistrModelMsh()
    meshHeaderData = pyFistr.getHeaderNumData(lines)
    return meshHeaderData

#
#  getNodesDict
#---------------
def getNodesDict(meshHeaderData):
    """ 節点Noをkeyとして、座標を求めるnodesDictを作成する。"""
    nodesDict = {}
    for headerData in meshHeaderData:
        header = headerData[0]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!NODE":
            data = headerData[1]
            for nodeData in data:
                nodeNo = nodeData[0]
                loc = nodeData[1:]
                nodesDict[nodeNo] = loc
    return nodesDict

#
#  getElementDict
#------------------
def getElementDict(meshHeaderData):
    """ 要素Noをkeyとして、節点列・要素typeが取得できる
    elementDictを作成する。"""
    elementDict = {}
    for headerData in meshHeaderData:
        header = headerData[0]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!ELEMENT":
            elType = pyFistr.getValName(header, "TYPE")
            data = headerData[1]
            for elmData in data:
                elNo = elmData[0]
                nodes = elmData[1:]
                elementDict[elNo] = [nodes, elType]
    return elementDict

#
#  clearSaveFolder
#------------------
def clearSaveFolder():
    """ folder内をクリアする"""
    #folderDir = currDir + "/" + SaveFolder
    folderDir = currDir + os.sep + SaveFolder
    if os.path.exists(folderDir) == True:
        shutil.rmtree(folderDir)
        os.mkdir(folderDir)
    else:
        os.mkdir(folderDir)

#
#  createNodeGrpVtkFiles
#-----------------------
def createNodeGrpVtkFiles(meshHeaderData, nodesDict):
    """ 節点group毎のvtkFileを作成する。"""
    #nodeGroupを取得
    nodeGrps = []
    for headerData in meshHeaderData:
        header = headerData[0]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!NGROUP":
            name = pyFistr.getValName(header, "NGRP")
            data = headerData[1]
            nodeGrps.append([name, data])
    #grp名をチェック。
    nodeGrps = checkSameGroupName(nodeGrps)
    #vtkを作成
    for nodeGrp in nodeGrps:
        #nodeGrp毎にvtkを作成
        name = nodeGrp[0]
        data = nodeGrp[1]
        vtkLines = getVtkNodeData(data, nodesDict)
        #file保存
        #fileName = currDir + "/" + SaveFolder + "/nodeGrp_" + name + ".vtu"
        fileName = currDir + os.sep + SaveFolder + os.sep + "nodeGrp_" + name + ".vtu"
        f = open(fileName, "w"); f.writelines(vtkLines); f.close()

#
#  getVtkNodeData
def getVtkNodeData(data, nodesDict):
    """ 節点GroupのvtkLinesを作成する。"""
    cont  = ['<?xml version="1.0"?>\n']
    cont += ['<VTKFile type="UnstructuredGrid" version="1.0">\n']
    cont += ['<UnstructuredGrid>\n']
    cont += ['<Piece NumberOfPoints="' + str(len(data)) + '" NumberOfCells="' + str(len(data)) + '">\n']
    cont += ['<Points>\n']
    cont += ['<DataArray type="Float32" NumberOfComponents="3" format="ascii">\n']
    for nodeNo in data:
        loc = nodesDict[nodeNo]
        cont += [" ".join(map(str, loc)) + "\n"]
    cont += ['</DataArray>\n']
    cont += ['</Points>\n']
    cont += ['<Cells>\n']
    cont += ['<DataArray type="Int32" Name="connectivity" format="ascii">\n']
    for i in range(len(data)):
        cont += [str(i) + "\n"]
    cont += ['</DataArray>\n']
    cont += ['<DataArray type="Int32" Name="offsets" format="ascii">\n']
    for i in range(len(data)):
        cont += [str(i+1) + "\n"]
    cont += ['</DataArray>\n']
    cont += ['<DataArray type="UInt8" Name="types" format="ascii">\n']
    for i in range(len(data)):
        cont += ['1\n']
    cont += ['</DataArray>\n']
    cont += ['</Cells>\n']
    cont += ['</Piece>\n']
    cont += ['</UnstructuredGrid>\n']
    cont += ['</VTKFile>\n']
    return cont

#
#  getVtkSurfaceData
def getVtkSurfaceData(surData, nodeDict):
    """ 表面GroupのvtkLinesを作成する。"""
    #surNodeDict作成
    surNodeDict = {}
    for surNodes, _name in surData:
        for surNode in surNodes:
            surNodeDict[surNode] = 0
    nodes = list(surNodeDict.keys())
    nodes.sort()
    for i in range(len(nodes)):
        nodeNo = nodes[i]
        surNodeDict[nodeNo] = i     #newNodeNoを設定

    lines  = ['<?xml version="1.0"?>\n']
    lines += ['<VTKFile type="UnstructuredGrid" version="1.0">\n']
    lines += ['<UnstructuredGrid>\n']
    nPoints = len(nodes)
    nCells = len(surData)
    lines += ['<Piece NumberOfPoints="' + str(nPoints) + '" NumberOfCells="' + str(nCells) + '">\n']
    lines += ['<Points>\n']
    lines += ['<DataArray type="Float32" NumberOfComponents="3" format="ascii">\n']
    for nodeNo in nodes:
        loc = nodeDict[nodeNo]
        lines += [" ".join(map(str, loc)) + "\n"]
    lines += ['</DataArray>\n']
    lines += ['</Points>\n']
    lines += ['<Cells>\n']
    lines += ['<DataArray type="Int32" Name="connectivity" format="ascii">\n']
    for surNodes, vtkName in surData:
        newNodes = list(map(lambda x: surNodeDict[x], surNodes))
        lines += [" ".join(map(str, newNodes)) + "\n"]
    lines += ['</DataArray>\n']
    lines += ['<DataArray type="Int32" Name="offsets" format="ascii">\n']
    n = 0
    for surNodes, vtkName in surData:
        n += len(surNodes)
        lines += [str(n) + "\n"]
    lines += ['</DataArray>\n']
    lines += ['<DataArray type="UInt8" Name="types" format="ascii">\n']
    for surNodes, vtkName in surData:
        lines += vtkName + "\n"
    lines += ['</DataArray>\n']
    lines += ['</Cells>\n']
    lines += ['</Piece>\n']
    lines += ['</UnstructuredGrid>\n']
    lines += ['</VTKFile>\n']
    return lines

#
#  createSurfaceGrpVtkFiles
#----------------------------
def createSurfaceGrpVtkFiles(meshHeaderData, nodesDict, elementDict):
    """ 表面group名毎にvtkFileを作成する。"""
    #surfaceGroupを作成
    surfaceGrps = []
    for headerData in meshHeaderData:
        header = headerData[0]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!SGROUP":
            name = pyFistr.getValName(header, "SGRP")
            data = headerData[1]
            surfaceGrps.append([name, data])
    #grp名をチェック。
    surfaceGrps = checkSameGroupName(surfaceGrps)
    #vtkを作成
    for surfaceGrp in surfaceGrps:
        #surface毎にvtkを作成
        name = surfaceGrp[0]
        data = surfaceGrp[1]
        surData = getSurfaceDataFromFaceNo(data, elementDict)
        vtkLines = getVtkSurfaceData(surData, nodesDict)
        #file保存
        #fileName = currDir + "/" + SaveFolder + "/surfaceGrp_" + name + ".vtu"
        fileName = currDir + os.sep + SaveFolder + os.sep + "surfaceGrp_" + name + ".vtu"
        f = open(fileName, "w"); f.writelines(vtkLines); f.close()

#
#  getSurfaceDataFromFaceNo
def getSurfaceDataFromFaceNo(data, elementDict):
    """ 要素のfaceNoから節点No列を取得する。"""
    faceTypeDict = {
      #node数 要素type
        3:  "231",
        4:  "241",
        6:  "232",
        8:  "242"
        }
    surData = []
    for i in range(0, len(data), 2):
        (elNo, faceNo) = data[i:i+2]
        elCont = elementDict[elNo]
        elNodes = elCont[0]
        elType = elCont[1]
        faceNodes = cm.fistr_elFaceNode(elType, elNodes, faceNo)
        faceType = faceTypeDict[len(faceNodes)]
        (vtkName, vtkNodes) = cm.fistr2vtk_el(faceType, faceNodes)
        surData.append([vtkNodes, vtkName])
    return surData

#
#  checkSameGroupName
#---------------------
def checkSameGroupName(Grps):
    """ 同じgroup名があるかチェック。あれば、まとめる。"""
    newGrps = []
    for grp in Grps:
        name = grp[0]
        data = grp[1]
        flag = 0
        for newGrp in newGrps:
            if newGrp[0] == name:
                flag = 1
                #同じgrpNameあり
                newGrp[1] += data
                break
        if flag == 0:
            newGrps.append(grp)
    return newGrps

#
#  createElementGrpVtkFiles
#--------------------------
def createElementGrpVtkFiles(meshHeaderData, nodesDict, elementDict):
    """ 要素group毎にvtkFileを作成する"""
    #elementGroupを作成
    elementGrps = []
    #!ELEMENTを作成
    for headerData in meshHeaderData:
        header = headerData[0]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!ELEMENT":
            name = pyFistr.getValName(header, "EGRP")
            data = headerData[1]
            elementGrps.append([name, data])
    #同じgrpNameをチェック、あればまとめる
    elementGrps = checkSameGroupName(elementGrps)
    #vtkを作成
    allElmData = []
    for elementGrp in elementGrps:
        #element毎にvtkを作成
        name = elementGrp[0]
        data = elementGrp[1]
        elmData = getElementDataForVtk(data, elementDict)
        allElmData += elmData
        vtkLines = getVtkElementData(elmData, nodesDict)
        #file保存
        #fileName = currDir + "/" + SaveFolder + "/elementGrp_" + name + ".vtu"
        fileName = currDir + os.sep + SaveFolder + os.sep + "elementGrp_" + name + ".vtu"
        f = open(fileName, "w"); f.writelines(vtkLines); f.close()
    #allElmDataでvtkを作成
    vtkLines = getVtkElementData(allElmData, nodesDict)
    #fileName = currDir + "/" + SaveFolder + "/allElement.vtu"
    fileName = currDir + os.sep + SaveFolder + os.sep + "allElement.vtu"
    f = open(fileName, "w"); f.writelines(vtkLines); f.close()
    
    #!EGROUPを作成
    elementGrps = []
    for headerData in meshHeaderData:
        header = headerData[0]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!EGROUP":
            name = pyFistr.getValName(header, "EGRP")
            data = []
            for elNo in headerData[1]:
                nodes = elementDict[elNo][0]
                elm = [elNo] + nodes
                data.append(elm)
            elementGrps.append([name, data])
    #vtkを作成
    for elementGrp in elementGrps:
        #element毎にvtkを作成
        name = elementGrp[0]
        data = elementGrp[1]
        elmData = getElementDataForVtk(data, elementDict)
        vtkLines = getVtkElementData(elmData, nodesDict)
        #file保存
        #fileName = currDir + "/" + SaveFolder + "/elementGrp_" + name + ".vtu"
        fileName = currDir + os.sep + SaveFolder + os.sep + "elementGrp_" + name + ".vtu"
        f = open(fileName, "w"); f.writelines(vtkLines); f.close()


#
#  getElementDataForVtk
def getElementDataForVtk(data, elementDict):
    """ fistr形式の要素データをvtk形式の要素データに変換する。"""
    elmData = []
    for i in range(len(data)):
        elNo = data[i][0]
        nodes = data[i][1:]
        elCont = elementDict[elNo]
        elType = elCont[1]
        (vtkName, vtkNodes) = cm.fistr2vtk_el(elType, nodes)
        elmData.append([vtkNodes, vtkName])
    return elmData

#
#  getVtkElementData
def getVtkElementData(elmData, nodesDict):
    """ 要素group毎のvtkファイルを作成する。"""
    #elmNodeDictを作成
    elmNodeDict = {}
    for nodes, _name in elmData:
        for node in nodes:
            elmNodeDict[node] = 0
    nodes = list(elmNodeDict.keys())
    nodes.sort()
    for i in range(len(nodes)):
        nodeNo = nodes[i]
        elmNodeDict[nodeNo] = i     #newNodeNoを設定

    lines  = ['<?xml version="1.0"?>\n']
    lines += ['<VTKFile type="UnstructuredGrid" version="1.0">\n']
    lines += ['<UnstructuredGrid>\n']
    nPoints = len(nodes)
    nCells = len(elmData)
    lines += ['<Piece NumberOfPoints="' + str(nPoints) + '" NumberOfCells="' + str(nCells) + '">\n']
    lines += ['<Points>\n']
    lines += ['<DataArray type="Float32" NumberOfComponents="3" format="ascii">\n']
    for nodeNo in nodes:
        loc = nodesDict[nodeNo]
        lines += [" ".join(map(str, loc)) + "\n"]
    lines += ['</DataArray>\n']
    lines += ['</Points>\n']
    lines += ['<Cells>\n']
    lines += ['<DataArray type="Int32" Name="connectivity" format="ascii">\n']
    for elmNodes, vtkName in elmData:
        newNodes = list(map(lambda x: elmNodeDict[x], elmNodes))
        lines += [" ".join(map(str, newNodes)) + "\n"]
    lines += ['</DataArray>\n']
    lines += ['<DataArray type="Int32" Name="offsets" format="ascii">\n']
    n = 0
    for elmNodes, vtkName in elmData:
        n += len(elmNodes)
        lines += [str(n) + "\n"]
    lines += ['</DataArray>\n']
    lines += ['<DataArray type="UInt8" Name="types" format="ascii">\n']
    for elmNodes, vtkName in elmData:
        lines += vtkName + "\n"
    lines += ['</DataArray>\n']
    lines += ['</Cells>\n']
    lines += ['</Piece>\n']
    lines += ['</UnstructuredGrid>\n']
    lines += ['</VTKFile>\n']
    return lines

#
#  createVtkFiles
#-----------------
def createVtkFiles(currDir):
    """ 要素、表面、節点group毎にvtkFileを作成する。"""
    os.chdir(currDir)
    #mesh読み込み
    meshHeaderData = getMeshHeaderData()
    nodesDict = getNodesDict(meshHeaderData)
    elementDict = getElementDict(meshHeaderData)
    clearSaveFolder()
    createNodeGrpVtkFiles(meshHeaderData, nodesDict)
    createSurfaceGrpVtkFiles(meshHeaderData, nodesDict, elementDict)
    createElementGrpVtkFiles(meshHeaderData, nodesDict, elementDict)


if __name__ == "__main__":
    import gettext
    gettext.install("app")
    _ = gettext.gettext
    if len(sys.argv) == 1:
        currDir = os.getcwd()
    else:
        currDir = sys.argv[1]
    currDir = os.path.abspath(currDir)
    createVtkFiles(currDir)
